--[[

Copyright 2008-∞ Daniel Ceregatti (daniel@ceregatti.org)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.

]]--

ProfileSwitcher = PitBull:NewModule("ProfileSwitcher", "LibRockDB-1.0", "LibRockEvent-1.0", "LibRockConfig-1.0")
local self = ProfileSwitcher
local localization = {}
local L = PitBull:L("PitBull-ProfileSwitcher", localization)
local delayCheck = 1
local delayTimer = 0
local dummyFrame
local lastZoneName = ""
local pitbullProfiles
self.desc = L["Switch PitBull profiles based on instance type."]
self:SetDatabase("ProfileSwitcherDB")

local BZ = Rock("LibBabble-Zone-3.0"):GetLookupTable()
local instanceKey = BZ["Alterac Valley"]

local instances = {}
instances["Alterac Valley"] = BZ["Alterac Valley"]
instances["Arathi Basin"] = BZ["Arathi Basin"]
instances["Black Temple"] = BZ["Black Temple"]
instances["Blade's Edge Arena"] = BZ["Blade's Edge Arena"]
instances["Dalaran Sewers"] = BZ["Dalaran Sewers"]
instances["Eye of the Storm"] = BZ["Eye of the Storm"]
instances["Gruul's Lair"] = BZ["Gruul's Lair"]
instances["Hyjal Summit"] = BZ["Hyjal Summit"]
instances["Karazhan"] = BZ["Karazhan"]
instances["Magtheridon's Lair"] = BZ["Magtheridon's Lair"]
instances["Nagrand Arena"] =BZ["Nagrand Arena"]
instances["Naxxramas"] = BZ["Naxxramas"]
instances["NaxxramasH"] = BZ["Naxxramas"]..L[" (Heroic)"]
instances["Ruins of Lordaeron"] = BZ["Ruins of Lordaeron"]
instances["Serpentshrine Cavern"] = BZ["Serpentshrine Cavern"]
instances["Strand of the Ancients"] = BZ["Strand of the Ancients"]
instances["Sunwell Plateau"] = BZ["Sunwell Plateau"]
instances["The Eye"] = BZ["The Eye"]
instances["The Eye of Eternity"] = BZ["The Eye of Eternity"]
instances["The Eye of EternityH"] = BZ["The Eye of Eternity"]..L[" (Heroic)"]
instances["The Obsidian Sanctum"] = BZ["The Obsidian Sanctum"]
instances["The Obsidian SanctumH"] = BZ["The Obsidian Sanctum"]..L[" (Heroic)"]
instances["The Ring of Valor"] = BZ["The Ring of Valor"]
instances["Ulduar"] = BZ["Ulduar"]
instances["UlduarH"] = BZ["Ulduar"]..L[" (Heroic)"]
instances["Vault of Archavon"] = BZ["Vault of Archavon"]
instances["Vault of ArchavonH"] = BZ["Vault of Archavon"]..L[" (Heroic)"]
instances["Wintergrasp"] = BZ["Wintergrasp"]
instances["Warsong Gulch"] = BZ["Warsong Gulch"]
instances["Zul'Aman"] = BZ["Zul'Aman"]

--[[

Taken verbatim (mostly) from LibRockDB-1.0.lua, because GetProfileList is local (why?)

]]--
local MAJOR_VERSION = "LibRockDB-1.0"
local newList, unpackDictAndDel = Rock:GetRecyclingFunctions(MAJOR_VERSION, "newList", "unpackDictAndDel")
local _G = _G
local _,race = _G.UnitRace("player")
local faction
if race == "Orc" or race == "Scourge" or race == "Troll" or race == "Tauren" or race == "BloodElf" then
	faction = _G.FACTION_HORDE
else    
	faction = _G.FACTION_ALLIANCE
end
local server = _G.GetRealmName():trim()
local _,classID = _G.UnitClass("player")
classID = classID:sub(1, 1) .. classID:sub(2):lower() -- Warrior instead of WARRIOR
local charID = _G.UnitName("player") .. " - " .. server
local realmID = server .. " - " .. faction

local function GetProfileList(addon)
	local t = newList()
	t.char = L["Character: "] .. L["%s of %s"]:format(UnitName("player"), server)
	t.realm = L["Realm: "] .. realmID
	t.class = L["Class: "] .. classID
	t.Default = L["Default"]
	local addon_db = addon.db
	local db = addon_db and addon_db.raw
	if db then
		if db.profiles then
			for k in pairs(db.profiles) do
				if not k:find("^char/") and not k:find("^realm/") and not k:find("^class/") and not t[k] then
					t[k] = k
				end
			end
		end
		if db.namespaces then
			for _,n in pairs(db.namespaces) do
				if n.profiles then
					for k in pairs(n.profiles) do
						if not k:find("^char/") and not k:find("^realm/") and not k:find("^class/") and not t[k] then
							t[k] = k
						end
					end
				end
			end
		end
		local curr = db.currentProfile and db.currentProfile[charID]
		if curr and not t[curr] then
			t[curr] = curr
		end
	end
	if t.Alternative then
		t.Alternative = L["Alternative"]
	end
	return t --"@dict", unpackDictAndDel(t)
end

-- End of LibRockDB stuff

local globalSettings = {
	name = L["Profile Switcher"],
	desc = L["Profile Switcher options."],
	type = 'group',
	hidden = function() return not ProfileSwitcher:IsActive() end,
	args = {
		instanceName = {
			type = "choice",  
			name = L["Instances"],
			desc = L["The names of the instances to associate with a PitBull profile. Start by selecting an instance."],
			choices = function() return instances end,
			get = function(key)
				return instanceKey
			end,
			set = function(key, value)
				instanceKey = key
			end,
			order = 1,
		},
		profileName = {
			type = 'choice',
			name = L["PitBull Profiles"],
			desc = L["The name of the PitBull profiles to associate with an instance. Once an instance is selected above, select which profile to use in that instance."],
			choices = function()
				return "@dict", unpackDictAndDel(GetProfileList(PitBull))
			end,
			get = function(key)
				return ProfileSwitcher.db.profile.instances[instanceKey] or ProfileSwitcher.db.profile.defaultPitBullProfile
			end,
			set = function(key, value)
				ProfileSwitcher.db.profile.instances[instanceKey] = key
			end,
			order = 2,
		},
		defaultPitBullProfile= {
			type = 'choice',
			name = L["Default PitBull Profile"],
			desc = L["The name of the PitBull Profile to use as the default."],
			choices = function()
				return "@dict", unpackDictAndDel(GetProfileList(PitBull))
			end,
			get = function(key)
				return ProfileSwitcher.db.profile.defaultPitBullProfile
			end,
			set = function(key, value)
				ProfileSwitcher.db.profile.defaultPitBullProfile = key
			end,
			order = 3,
		},
	},
}
PitBull:RegisterGlobalSetting("PitBull", function() return "@cache", globalSettings end) 

function ProfileSwitcher:IsActive()
	return true
end

function ProfileSwitcher:OnInitialize()
	self.db = PitBull:GetDatabaseNamespace("ProfileSwitcher")
	PitBull:SetDatabaseNamespaceDefaults('ProfileSwitcher', 'profile', {
		instances = {},
		defaultPitBullProfile = 'Default'
	})
	-- Create our dummy frame so GetRealZoneText() can work properly
	dummyFrame = CreateFrame ("Frame")
	self:AddEventListener("PLAYER_ENTERING_WORLD", "PLAYER_ENTERING_WORLD")
	-- Get the list of available profiles from PitBull's db
	pitbullProfiles = GetProfileList(PitBull)
end

function ProfileSwitcher:PLAYER_ENTERING_WORLD()
	dummyFrame:SetScript ("OnUpdate", self.OnUpdate)
end

function ProfileSwitcher:OnUpdate(arg)
	delayTimer = delayTimer + arg
	if (delayTimer > delayCheck) then
		delayTimer = 0
		local realZoneName = GetRealZoneText()
		if (realZoneName ~= lastZoneName) then
			local difficulty = 1
			inInstance, instanceType = IsInInstance()
			if inInstance then
				--[===[@debug@
				pp (format ("I'm in an instance! %s", instanceType))
				--@end-debug@]===]
				difficulty = GetCurrentDungeonDifficulty()
			end
			local zoneName = realZoneName
			if difficulty == 2 then
			  zoneName = zoneName.."H"
			end
			local setZone = getPrefForZone(zoneName)
			if setZone == 'none' then
				--[===[@debug@
				pp ("Zone change detected, but there's no pref. Keeping existing zone")
				--@end-debug@]===]
			else
				PitBull:SetProfile(setZone)
				--[===[@debug@
				pp (format ("Zone change detected. New: %s, Real: %s, Old: %s, Profile: %s", zoneName, realZoneName, lastZoneName, setZone))
				--@end-debug@]===]
			end
			lastZoneName = realZoneName
			dummyFrame:SetScript ("OnUpdate", nil)
		end
	end
end

function getPrefForZone(zoneName)
	return ProfileSwitcher.db.profile.instances[zoneName] or 'none' 
end

function pp(text)
	DEFAULT_CHAT_FRAME:AddMessage (text)
end
